@@ -17,6 +17,11 @@
 ## Remove DLLs when package is unloaded
 
 # nocov start
+.onLoad <- function(libname, pkgname) {
+  # Scheme defaults are fairly complex...
+
+  check_assumptions()
+}
 
 .onUnload <- function(libpath) {
   library.dynam.unload("vetr", libpath)
